from collections.abc import Awaitable, Callable
from typing import Any, Protocol, TypeAlias

from asgiref.typing import ASGIApplication, ASGIReceiveCallable

def name_that_thing(thing: Any) -> str: ...
async def await_many_dispatch(
    consumer_callables: list[Callable[[], Awaitable[ASGIReceiveCallable]]],
    dispatch: Callable[[dict[str, Any]], Awaitable[None]],
) -> None: ...

class _MiddlewareProtocol(Protocol):
    def __init__(self, *args, **kwargs) -> None: ...
    async def __call__(self, scope: Any, receive: Any, send: Any) -> Any: ...

_ChannelApplication: TypeAlias = _MiddlewareProtocol | ASGIApplication  # noqa



from .application import ApplicationCommunicator
from .http import HttpCommunicator
from .live import ChannelsLiveServerTestCase
from .websocket import WebsocketCommunicator

__all__ = [
    "ApplicationCommunicator",
    "HttpCommunicator",
    "ChannelsLiveServerTestCase",
    "WebsocketCommunicator",
]



from typing import Any, TypeVar

from asgiref.typing import ASGIReceiveCallable, ASGISendCallable
from channels.middleware import BaseMiddleware
from django.contrib.auth.backends import BaseBackend
from django.contrib.auth.base_user import AbstractBaseUser
from django.contrib.auth.models import AnonymousUser
from django.utils.functional import LazyObject

from .consumer import _ChannelScope, _LazySession
from .db import database_sync_to_async
from .utils import _ChannelApplication

_User = TypeVar("_User", bound=AbstractBaseUser)

@database_sync_to_async  # type: ignore
def get_user(scope: _ChannelScope) -> _User | AnonymousUser: ...
@database_sync_to_async
def login(
    scope: _ChannelScope, user: _User, backend: BaseBackend | None = ...
) -> None: ...
@database_sync_to_async
def logout(scope: _ChannelScope) -> None: ...
def _get_user_session_key(session: _LazySession) -> Any: ...

class UserLazyObject(AbstractBaseUser, LazyObject):
    def _setup(self) -> None: ...

class AuthMiddleware(BaseMiddleware):
    def populate_scope(self, scope: _ChannelScope) -> None: ...
    async def resolve_scope(self, scope: _ChannelScope) -> None: ...
    async def __call__(
        self,
        scope: _ChannelScope,
        receive: ASGIReceiveCallable,
        send: ASGISendCallable,
    ) -> _ChannelApplication: ...

def AuthMiddlewareStack(inner: _ChannelApplication) -> _ChannelApplication: ...



from typing import Any, Sequence, TypeAlias

from asgiref.typing import ASGIReceiveCallable, ASGISendCallable
from django.urls.resolvers import URLPattern, URLResolver

from .consumer import _ChannelScope
from .utils import _ChannelApplication

def get_default_application() -> ProtocolTypeRouter: ...

class ProtocolTypeRouter:
    application_mapping: dict[str, _ChannelApplication]

    def __init__(self, application_mapping: dict[str, Any]) -> None: ...
    async def __call__(
        self,
        scope: _ChannelScope,
        receive: ASGIReceiveCallable,
        send: ASGISendCallable,
    ) -> None: ...

_IncludedURLConf: TypeAlias = tuple[
    Sequence[URLResolver | URLPattern], str | None, str | None
]

class URLRouter(_IncludedURLConf):
    _path_routing: bool = ...
    routes: list[URLPattern | URLResolver]

    def __init__(self, routes: list[URLPattern | URLResolver]) -> None: ...
    async def __call__(
        self,
        scope: _ChannelScope,
        receive: ASGIReceiveCallable,
        send: ASGISendCallable,
    ) -> None: ...

class ChannelNameRouter:
    application_mapping: dict[str, _ChannelApplication]

    def __init__(self, application_mapping: dict[str, _ChannelApplication]) -> None: ...
    async def __call__(
        self,
        scope: _ChannelScope,
        receive: ASGIReceiveCallable,
        send: ASGISendCallable,
    ) -> None: ...



from re import Pattern
from typing import Any, Iterable
from urllib.parse import ParseResult

from asgiref.typing import ASGIReceiveCallable, ASGISendCallable
from channels.consumer import _ChannelScope
from channels.generic.websocket import AsyncWebsocketConsumer
from channels.utils import _ChannelApplication

class OriginValidator:
    application: _ChannelApplication
    allowed_origins: Iterable[str | Pattern[str]]

    def __init__(
        self,
        application: _ChannelApplication,
        allowed_origins: Iterable[str | Pattern[str]],
    ) -> None: ...
    async def __call__(
        self,
        scope: _ChannelScope,
        receive: ASGIReceiveCallable,
        send: ASGISendCallable,
    ) -> Any: ...
    def valid_origin(self, parsed_origin: ParseResult | None) -> bool: ...
    def validate_origin(self, parsed_origin: ParseResult | None) -> bool: ...
    def match_allowed_origin(
        self, parsed_origin: ParseResult | None, pattern: str | Pattern[str]
    ) -> bool: ...
    def get_origin_port(self, origin: ParseResult | None) -> int | None: ...

def AllowedHostsOriginValidator(
    application: _ChannelApplication,
) -> OriginValidator: ...

class WebsocketDenier(AsyncWebsocketConsumer):
    async def connect(self) -> None: ...



from collections.abc import Callable
from typing import Any, ClassVar, TypeAlias

from channels.routing import ProtocolTypeRouter
from channels.utils import _ChannelApplication
from daphne.testing import DaphneProcess
from django.contrib.staticfiles.handlers import ASGIStaticFilesHandler
from django.db.backends.base.base import BaseDatabaseWrapper
from django.db.backends.sqlite3.base import DatabaseWrapper
from django.test.testcases import TransactionTestCase
from django.test.utils import modify_settings

_StaticWrapper: TypeAlias = Callable[[ProtocolTypeRouter], _ChannelApplication]

def make_application(*, static_wrapper: _StaticWrapper | None) -> Any: ...

class ChannelsLiveServerTestCase(TransactionTestCase):
    host: ClassVar[str] = ...
    ProtocolServerProcess: ClassVar[type[DaphneProcess]] = ...
    static_wrapper: ClassVar[type[ASGIStaticFilesHandler]] = ...
    serve_static: ClassVar[bool] = ...

    _port: int
    _server_process: DaphneProcess
    _live_server_modified_settings: modify_settings

    @property
    def live_server_url(self) -> str: ...
    @property
    def live_server_ws_url(self) -> str: ...
    def _pre_setup(self) -> None: ...
    def _post_teardown(self) -> None: ...
    def _is_in_memory_db(
        self, connection: BaseDatabaseWrapper | DatabaseWrapper
    ) -> bool: ...



from collections.abc import Iterable
from typing import Any

from asgiref.typing import (
    HTTPDisconnectEvent,
    HTTPRequestEvent,
    HTTPScope,
)
from channels.consumer import AsyncConsumer

class AsyncHttpConsumer(AsyncConsumer):
    body: list[bytes]
    scope: HTTPScope  # type: ignore[assignment]

    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    async def send_headers(
        self,
        *,
        status: int = ...,
        headers: Iterable[tuple[bytes, bytes]] | None = ...,
    ) -> None: ...
    async def send_body(self, body: bytes, *, more_body: bool = ...) -> None: ...
    async def send_response(self, status: int, body: bytes, **kwargs: Any) -> None: ...
    async def handle(self, body: bytes) -> None: ...
    async def disconnect(self) -> None: ...
    async def http_request(self, message: HTTPRequestEvent) -> None: ...
    async def http_disconnect(self, message: HTTPDisconnectEvent) -> None: ...
    async def send(  # type: ignore[override]
        self,
        message: dict[str, Any],
    ) -> None: ...
